@@ -1,6 +1,6 @@
 package swat;
 
-our $VERSION = '0.1.87';
+our $VERSION = '0.1.88';
 
 use base 'Exporter'; 
 
@@ -38,26 +38,15 @@ sub config {
     return $config;
 }
 
-sub execute_with_retry {
-
-    my $cmd = shift;
-    my $try = shift || 1;
-
-    for my $i (1..$try){
-        note("\nexecute cmd: $cmd\n attempt number: $i") if debug_mod2();
-        return $i if system($cmd) == 0;
-        sleep $i**2;
-    }
-
-    return
-
-}
-
 sub make_http_request {
 
     return if get_prop('response_done');
 
     my ($fh, $content_file) = tempfile( DIR => get_prop('test_root_dir') );
+    
+    my $try_i;
+
+    my $try = get_prop('try_num');
 
     if (get_prop('response') and @{get_prop('response')} ){
 
@@ -82,28 +71,47 @@ sub make_http_request {
         my $resource = get_prop('resource');
         my $http_method = get_prop('http_method'); 
 
-        $curl_cmd.=' -f'  unless ignore_http_err();
-
-        my $curl_runner = "$curl_cmd -D $content_file.hdr -o $content_file --stderr $content_file.stderr '$hostname$resource'";
+        my $curl_runner = "$curl_cmd -w '%{response_code}' -D $content_file.hdr -o $content_file --stderr $content_file.stderr '$hostname$resource' > $content_file.http_status";
         my $curl_runner_short = "$curl_cmd -D - '$hostname$resource'";
+        my $http_status = 0;
 
-        note('trying ... '.$curl_runner_short) ;
+        TRY: for my $i (1..$try){
+            note("try N [$i] $curl_runner") if debug_mod12();
+            $try_i = $i;
+            system($curl_runner);
+            if(open HTTP_STATUS, "$content_file.http_status"){
+                $http_status = <HTTP_STATUS>;
+                close HTTP_STATUS;
+                chomp $http_status;
+                note("got http status: $http_status") if debug_mod12();
+                last TRY if $http_status < 400 and $http_status > 0;
+                last TRY if $http_status >= 400 and ignore_http_err();
 
-        my $st = execute_with_retry("$curl_runner && test -f $content_file.hdr", get_prop('try_num'));
+            }
+            my $delay = ($i)**2;
+            note("sleep for $delay seconds before next try") if debug_mod12();
+            sleep $delay; 
 
-        if ($st) {
+        }
 
-            ok(1, "server returned successful response");
+            
+        #note($curl_runner);
+
+        if ( $http_status < 400 and $http_status > 0 ) {
+
+             ok(1, "$http_status / $try_i of $try ".$curl_runner_short);
 
         }elsif(ignore_http_err()){
 
-            ok(1, "server returned bad response, we still continue due to ignore_http_err set to 1");
+            ok(1, "$http_status / $try_i of $try ".$curl_runner_short);
+            note("server returned bad response, we still continue due to ignore_http_err set to 1");
 
         }else{
 
-            ok(0, "server returned successful response");
+            ok(1, "$http_status / $try_i of $try ".$curl_runner_short);
 
             note "stderr:";
+
             open CURL_ERR, "$content_file.stderr" or die $!;
             while  ( my $i = <CURL_ERR>){
                 chomp $i;
@@ -201,6 +209,8 @@ sub generate_asserts {
 
     dsl()->{match_l} = get_prop('match_l');
 
+    return if http_method() eq 'META';
+
     make_http_request();
 
     dsl()->{output} = headers().body();
@@ -223,6 +233,17 @@ sub generate_asserts {
 
 }
 
+sub print_meta {
+
+    note('@'.http_method());
+    open META, resource_dir()."/meta.txt" or die $!;
+    while (my $i = <META>){
+        note("\t $i");
+    }
+    close META;
+    
+}
+
 1;
 
 
@@ -404,6 +425,8 @@ Here is the list of I<predefined> file names for a http methods files:
     put.txt      --> PUT      method
     delete.txt   --> DELETE   method
 
+There is also special predefined file name `meta.txt', see L<meta stories|#meta-stories>.
+
 
 =head1 Hostname / IP Address
 
@@ -752,30 +775,23 @@ For example:
 
 =item *
 
-C<ignore_http_err> - run curl with or without `-f' flag
+C<ignore_http_err> - ignore http related errors; default value is `1' ( do not ignore ) 
 
 =over
 
 =item *
 
-swat runs curl command and then checks exit code returned
+swat runs curl command and then checks http status code
 
 
 =item *
 
-if not successful exit code returns a proper swat assert fails
+in case of bad http status code returns (  0 or >= 400 ) a proper swat assert fails
 
 
 =item *
 
-by default ignore_http_err is set to 0 which make curl run with `-f' flag, for not successful http responses ( bad http status code )
-  this means error on swat side ( see previous point )
-
-
-=item *
-
-settings ignore_http_err to `1' make curl run curl without `-f' flag which result in even non successful http responses are not treated as errors 
-( a proper swat assert does not fail )
+settings ignore_http_err to `1' only makes a warning in case of bad http status code and does not trigger assert failure 
 
 
 =back
@@ -1404,7 +1420,52 @@ config() - returns hash of test suite configuration
 
 See L<test suite ini file|#test-suite-ini-file> section for details.
 
-Be aware of that these functions are readers not setters.
+Be aware of that these functions are readers.
+
+
+=head2 meta stories
+
+Meta stories are special type of swat stories. 
+
+The essential property of meta story is it has no related http request:
+
+    # foo/bar story
+    mkdir foo/bar
+    
+    # it's a meta story
+    touch foo/bar/meta.txt
+
+As meta story does not have any related http request, it has no `http method' file either.
+
+A `meta.txt' file should be treated as meta request file denoted that story is meta.
+
+You may live `meta.txt' empty file or add some useful description to be printed  when story is executed:
+
+    nano foo/bar/meta.txt
+    
+        This is my cool story. 
+        Take a look at this!
+
+How one could use meta stories?
+
+Meta stories are just I<containers> for other downstream stories. Usually one defines some downstream
+stories call inside meta story's hook file:
+
+    nano foo/bar/hook.pm
+    
+        run_swat_module( POST => '/baz' );
+        run_swat_module( POST => '/baz/bar' );
+
+Meta stories are very close to upstream stories with spoofed server response, 
+with the only exclusion that as meta story has no real http request related to it, 
+there is no need for spoofing.
+
+Meta stories can be also called as downstream stories:
+
+    # I am downstream story
+    # you can call me from somewhere else
+    nano foo/bar/swat.ini
+        swat_module=1
 
 
 =head2 PERL5LIB
